/*--------------------------------------------------------------------------
NETREK II -- Paradise

Permission to use, copy, modify, and distribute this software and its
documentation for any NON-COMMERCIAL purpose (following the terms of
the GNU General Public License (read the file 'COPYING')) and without
fee is hereby granted, provided that this copyright notice appear in all
copies.  No representations are made about the suitability of this
software for any purpose.  This software is provided "as is" without
express or implied warranty.

    Xtrek Copyright 1986                            Chris Guthrie
    Netrek (Xtrek II) Copyright 1989                Kevin P. Smith
                                                    Scott Silvey
    Paradise II (Netrek II) Copyright 1993          Larry Denys
                                                    Kurt Olsen
                                                    Brandon Gillespie

    Comprehensive credits are available in the file "CREDITS"
--------------------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include "struct.h"
#include "shmem.h"
#include "terrain.h"

#define MAXALTITUDE 255
#define MAXSTARS 20
#define X 0
#define Y 1

void generate_terrain()
/* Generates terrain based on specs in the system configuration.
   Places a number of "seeds" into the terrain grid, and then generates an
   "altitude" map based on those seeds' positions (which correspond to
   star positions).  Place_nebula generates another altitude map for 
   nebulae -- the first map is used for asteroids and generating the 
   second map.

   This function is called within the galaxy generation stuff.

   10/26/94 MM */
{
  int i,j,k;  /* counters */
  int x,y;    /* more counters, different purpose */
  double dist, val;   /* for distance calculations */
  int num_seeds = 0;
  int seed_xy[MAXSTARS][2];
  int qx[4] = {-1,1,-1,1}, qy[4] = {1,1,-1,-1}; /* quadrant multipliers */

/* place seeds -- this would be easy to change if you just had a number
   of seeds you wanted to place, instead of basing it on stars.  I won't
   bother doing it, even though it might be cool to see it work in a bronco
   game...  MM*/

  for (i=0; i < NUMPLANETS; i++)
    if (PL_TYPE(planets[i]) == PLSTAR) {
      terrain_grid[(planets[i].pl_x/TGRID_GRANULARITY) * TGRID_SIZE +
	           planets[i].pl_y/TGRID_GRANULARITY].alt1 = MAXALTITUDE;
      
      seed_xy[num_seeds][X] = planets[i].pl_x/TGRID_GRANULARITY;
      seed_xy[num_seeds][Y] = planets[i].pl_y/TGRID_GRANULARITY;
      num_seeds++;
    }

/* generate terrain -- simple, stupid version. */

  
  for (x=0; x < TGRID_SIZE; x++)
    for (y=0; y < TGRID_SIZE; y++) 
      if (terrain_grid[x*TGRID_SIZE + y].alt1 != MAXALTITUDE) {
	val = 0.0;
	for (i=0; i < num_seeds; i++) {
	  dist = (double)MAXALTITUDE -
	    sqrt((double)((x - seed_xy[i][X]) * (x - seed_xy[i][X]) +
		 (y - seed_xy[i][Y]) * (y - seed_xy[i][Y])));
	  if (dist > val) val = dist;
	}
	terrain_grid[x * TGRID_SIZE + y].alt1 = (int)val;
      }
  
/* place nebula */

/* place asteroids */
/*  if (num_asteroid)*/  place_asteroids(0, 243, 0);
}

void place_nebula(int num_nebula, int num_seeds, int minalt)
/* Values inbetween MAXALTITUDE and minalt are considered nebulous terrain.
   Tries to cluster seeds in groups based on num_nebula and num_seeds.
   ...the number of seeds per nebula being num_seeds. 
   10/26/94 MM */
{
  
}

void place_asteroids(int density, int altitude, int num_fields)
/* Marks terrain grid locations within density of altitude as asteroid
   fields.  I may make the chance of such a grid location becoming a
   field random (like 90% or something), so that fields will appear less
   uniform, and holes may exist in them.  Makes for interesting terrain, IMO.
   10/26/94 MM */
{
  int x,y,i,j,numstars=0;
  int *systems_with_asteroids;
  int star_numbers[NUMPLANETS];

  for (i=0; i < NUMPLANETS; i++)
     if (PL_TYPE(planets[i]) == PLSTAR) {
	star_numbers[numstars] = i;
	numstars++;
     }
  systems_with_asteroids = (int *)malloc(numstars*sizeof(int));
  for (i=0; i < numstars; i++) systems_with_asteroids[i] = 0;

/* assign what stars have asteroid belts -- I could just start with system #1,
   since systems are placed randomly, but I might as well pick a random system
   to start with.  The only prereq is that the system does NOT belong to a
   race. */

  if (*num_asteroid > numstars) *num_asteroid = numstars;

  for (i = 0, j = lrand48()%numstars; i < *num_asteroid; i++) {
     systems_with_asteroids[j] = 1;
     if (j == numstars-1) j = 0;
     else j++;
  }

  for (i=0; i < numstars; i++)
     if (systems_with_asteroids[i])
	printf("System %s has an asteroid belt\n", planets[star_numbers[i]].pl_name);

  for (x=0; x < TGRID_SIZE; x++)
    for (y=0; y < TGRID_SIZE; y++)
       for (i=0; i < numstars; i++) {
	  /* if the tgrid locale is within a certain distance of a system
	     that is supposed to have an asteroid belt, then, AST_CHANCE
	     percent of the time, mark that locale as having asteroids */
	  if (systems_with_asteroids[i] &&
	      terrain_grid[x * TGRID_SIZE + y].alt1 >= altitude-(*asteroid_thickness) && 
	      terrain_grid[x * TGRID_SIZE + y].alt1 <= altitude+(*asteroid_thickness) &&
	      (ABS(x*TGRID_GRANULARITY - planets[star_numbers[i]].pl_x) <
	       (MAXALTITUDE-(altitude-(*asteroid_thickness+1))) * TGRID_GRANULARITY) &&
	      (ABS(y*TGRID_GRANULARITY - planets[star_numbers[i]].pl_y) <
	       (MAXALTITUDE-(altitude-(*asteroid_thickness+1))) * TGRID_GRANULARITY) &&
	      lrand48()%100 >= AST_CHANCE)
	     terrain_grid[x * TGRID_SIZE + y].types |= T_ASTEROIDS;
       }
}
